/* (C) 2021 YiRing, Inc. */
package com.yiring.common.param;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import java.io.Serial;
import java.io.Serializable;
import java.util.Objects;
import javax.validation.constraints.Min;
import lombok.AccessLevel;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.FieldDefaults;
import lombok.experimental.SuperBuilder;
import org.hibernate.validator.constraints.Range;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

/**
 * 可选的公共的分页排序查询参数
 *
 * @author ifzm
 * @version 0.1 2019/3/10 16:29
 */
@ApiModel(value = "OptionalPageParam", description = "可选的公共的分页排序查询参数")
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
@FieldDefaults(level = AccessLevel.PRIVATE)
public class OptionalPageParam implements Serializable {

    @Serial
    private static final long serialVersionUID = 6103761701912769946L;

    @ApiModelProperty(value = "分页条数", example = "10", required = true)
    @Range(min = 1, max = 100)
    Integer pageSize;

    @ApiModelProperty(value = "当前页数", example = "1", required = true)
    @Min(1)
    Integer pageNo;

    @ApiModelProperty(value = "排序字段", example = "id")
    String sortField;

    @ApiModelProperty(value = "排序方向（ASC|DESC）", example = "DESC")
    Sort.Direction sortOrder;

    /**
     * 根据参数构建分页对象
     *
     * @param param {@link OptionalPageParam}
     * @return Pageable
     */
    @SuppressWarnings({ "unused" })
    public static Pageable toPageable(OptionalPageParam param) {
        if (param == null) {
            return Pageable.unpaged();
        }

        Sort sort = Sort.unsorted();
        if (Objects.nonNull(param.getSortField())) {
            sort = Sort.by(new Sort.Order(param.getSortOrder(), param.getSortField()));
        }
        return PageRequest.of(param.getPageNo() - 1, param.getPageSize(), sort);
    }
}
